function [a,t] = aufwand(expr,model)
% [a,t] = aufwand(expr,model)
% Kostenabschtzung der Realisierung von Schaltnetzen.
%
% expr :  String des vollstndig geklammerten schaltalgebraischen Ausdrucks 
%         in MATLAB-Syntax (berzhlige Klammern sind verboten).
%         CellArray von Strings aller Zwischenrechnungen und Ausgaben im 
%         Falle von Schaltnetzen
%
% Die Kosten von AND-, NAND-, OR-, NOR-, XOR- und XNOR-Gattern sind in einer 
% 3*5-Matrix  model  festgelegt:
% Die drei Zeilen stehen der Reihe nach fr UND, ODER und XOR, die Spalten fr:
% Hardware direkt pro Gatter / Hardware pro Gatter wenn Ausgang negiert / 
% Hardware pro Gattereingang / Zeit direkt / Zeit wenn Gatterausgang negiert.
% Passende Matrizen c sind whlbar durch  c = 'MOS'  bzw.  c = 'bipolar' .
% Die Modelle M1/Z1 und M2/Z2 sind durch c = 'M1' bzw. c = 'M2' whlbar.
% Default: c = 'MOS'
%
% Beispiele:  aufwand('~a&(b|(c&~e&f)|d)')
%             aufwand({'h=~a|(b&c)'; 'x=h|a'; 'y=h&d'})

% Autor:  Klaus von der Heide  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  help aufwand; return; end
if nargin<2  model = 'M2'; end
if ischar(model)
   switch upper(model)
      case {'GATECOUNT','M1'},  c = [1 1 0 1 1; 1 1 0 1 1; 1 1 0 1 1];
      case 'M2',                c = [0 0 1 1 1; 0 0 1 1 1; 0 0 1 1 1];
      case 'MOS',               c = [1 0 1.25 2 1; 1 0 1.25 2 1; 0 0 2 2 2];
      case 'BIPOLAR',           c = [1 1 0 1 1; 1 1 0 1 1; 1 1 0 1 1];
   end
else
   c = model;
end

% Aufwand
% =======
if iscell(expr)                       % falls Schaltnetz, alle Ausdrcke 
   for k=1:length(expr)
      [aa(k),tt(k)] = aufwand(expr{k},c); 
   end
   a = sum(aa);
   t = max(tt);
else
   [a,t] = kosten(binexpr2tree(expr),c);
end

% Ausgabe
% =======
if ~nargout
   if ischar(model)
      Aufwand{1} = ['Modell: ' model];
   else
      Aufwand{1} = ['Modell: custom'];
   end
   Aufwand{2} = ['Aufwandsmass:   '  num2str(a)];
   Aufwand{3} = ['Gatterlaufzeit: '  num2str(t)];
   result2fig(Aufwand)
end

%-----------------------------------------------------------------------

function [a,t] = kosten(tree,c)      % Kostenfunktion fr Schaltfunktionen
if ~iscell(tree) 
   a = 0;                            % Kosten der Identitt 
   t = 0;
else
   nop = length(tree) - 1;           % Anzahl der Operanden
   op  = tree{1};                    % Operation
   switch op
      case {'&','*'},    a = c(1,1) + nop*c(1,3);  t = c(1,4); % AND
      case {'/&','/*'},  a = c(1,2) + nop*c(1,3);  t = c(1,5); % NAND
      case {'|'},        a = c(2,1) + nop*c(2,3);  t = c(2,4); % OR
      case {'/|'},       a = c(2,2) + nop*c(2,3);  t = c(2,5); % NOR
      case {'+'},        a = c(3,1) + nop*c(3,3);  t = c(3,4); % XOR
      case {'/+'},       a = c(3,2) + nop*c(3,3);  t = c(3,5); % EQUIV
   end
   for k=2:length(tree)
      [aa(k),tt(k)] = kosten(tree{k},c);  % rekursiv in den Baum 
   end
   a = a + sum(aa);
   t = t + max(tt);
end

