function [bestv,mdist,udist,zdist] = decoddemo(g,u,sigma)
% [bestv,mdist,udist,zdist] = decoddemo(g,u,sigma)
% Simulation einer Codierung und Decodierung fr lineare binre Blockcodes. 
% Decodierung durch vollstndige Suche nach minimalem euklidischen Abstand.
%
%   g   :  Generatormatrix des Blockcodes (Nullen und Einsen)
%   u   :  zu codierende Informationsbits (maximal 16)
% sigma :  Standardabweichung des Rauschens bezogen auf den Abstand 1.0 
%          der beiden Signalniveaus
%
% bestv :  dekodierte Information
% mdist :  Abstandsquadrat des Empfangswortes vom zugehrigen Kodewort
% udist :  Abstandsquadrat des Empfangswortes vom korrekten Kodewort
% zdist :  Abstandsquadrat des Empfangswortes vom zweitnchsten Kodewort
%
% Die Abstandsquadrate sind normiert auf den Abstand 1 zwischen den 
% beiden Niveaus. Im Fall  mdist<udist  liegt ein Dekodierfehler vor.
%
% Es werden drei Bilder gezeigt:
% 1. Das Abstandsquadrat der Kodewrter smtlicher Bitmuster 
%    derselben Lnge wie u vom simulierten Sendesignal
% 2. Die kumulative Verteilung der Abstandsquadrate
% 3. Die Verteilung der Abstandsquadrate ber dem Hammingabstand 
%    zwischen u und dem jeweiligen Bitmuster
% Wenn sigma > 0 , gibt es ein weiteres Bild mit der Verteilung der 
% empfangenen Werte.
%
% Beispiel: 
% decoddemo( randb([10,32]), randb(10), 0.25)

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3  sigma = 0.5;           end
if nargin<2      u = randb(8);      end
if nargin<1      g = randb([8,32]); end
if length(u)>16  
   warning('Vektor u nach 16 Bits abgeschnitten!'); 
   u = u(1:16);
end

% Konstanten
% ==========
[ngz,ngs] = size(g);                 % (ngs,ngz)-Code
n   = length(u);                     % Lnge des Bitmusters
if n~=ngz  error('Informationsvektor inkompatibel mit Generatormatrix'); end
del = ceil(5*sigma)/10;              % Auflsung der Fehlerverteilung
sgr = 1 + 7*del;                     % Darstellungsbereich der Verteilung
fsz = 13;                            % FontSize fr Darstellung
dm  = ngs/2 + 0.25*ngs*sigma^2;      % Erwartungswert der Distanz
dm2 = floor(2*dm);                   % 2*mittlere Distanz als Darstellungsgrenze
sm  = sqrt(ngs/2 + 0.5*ngs*sigma^2); % Erwartungswert der Standardabweichung
s   = [-1 +1];                       % Signalniveaus (sigma bezieht sich auf Abstand 1.0 !)

% Codierung
% =========
x = s(rem(u*g,2) + 1);

% Kanalrauschen
% =============
z = x + sigma*randn(size(x)); 

% Berechnung des Abstandes fr alle Bitmuster
% ===========================================
for k=1:2^n
   symb = s(1 + rem(num2binvec(k-1,n)*g,2)); % Codewort
   diff = z - symb;
   dist(k) = diff*diff';       % Abstandsquadrat dieses Musters vom Empfangswort
end
dist = dist/4;                 % Normierung auf Niveauabstand 1
sdist= sort(dist);

% Ausgabe
% =======
mdist = sdist(1);
udist = dist(binvec2num(u)+1);
bestk = min(find(dist==mdist))-1;
cabst = min(dist(find(dist>mdist))) - mdist;
zdist = cabst + mdist;
bestv = num2binvec(bestk,n);
muster= binmatout(bestv);

korrekt     = binmatout(u)
dekodiert   = muster
Kodeabstand = cabst

% Darstellungen
% =============
% Abstandsquadrat ber der Ganzzahlinterpretation des Bitmusters
if sigma<0.01
   f1pos = [2 1; 2 1];
else
   f1pos = [2 2; 2 1];
% Fehlerverteilung
   fh2 = fullscreen('Verteilung','w',0,[2 2; 2 2]);
   axes('Position',[0.07 0.13 0.88 0.8])
   hist(z,-sgr:del:+sgr)           % Darstellung der Fehlerverteilung
   n1  = sum(flach(x)==+1);        % Anzahl der +1
   n0  = sum(flach(x)==-1);        % Anzahl der -1
   s   = -sgr:del/20:+sgr;         % Darstellungsbereich
   yd  = 0.00005*n*ngs;            % vertikale Verschiebung einer Linie
   g1  = exp(-((s-1).^2)/(2*sigma^2))*n1*del/(sqrt(2*pi)*sigma); % Gauss-Verteilung
   g0  = exp(-((s+1).^2)/(2*sigma^2))*n0*del/(sqrt(2*pi)*sigma); % Gauss-Verteilung
   line(s,g0+g1+yd,'Color','w')    % Sichtbarmachen im Histogramm
   line(s,g0+g1,'Color','k')       % Summenverteilung
   line(s,g1,'Color','r')          % Gauss-Verteilung der +1-Signale
   line(s,g0,'Color','g')          % Gauss-Verteilung der -1-Signale
   xlabel('Verteilungen des Signalwertes;  rot: binre 1;  grn: binre 0;  schwarz/weiss: Summe;  blau: aktuelle Stichprobe','FontSize',10)
   ax = axis;
   text(0.88*ax(1),0.91*ax(4),sprintf('(%4d,%2d)-Code',ngs,ngz),'FontSize',fsz)
   text(0.88*ax(1),0.85*ax(4),sprintf('Code-Rate:            %1.4f',ngz/ngs),'FontSize',fsz)
   text(0.88*ax(1),0.80*ax(4),sprintf('Sigma:                   %1.1f',sigma),'FontSize',fsz)
   text(0.88*ax(1),0.75*ax(4),sprintf('Informationsbits:   %4d',n),'FontSize',fsz)
end
fh1 = fullscreen('Distanz','w',0,f1pos);
axes('Position',[0.1 0.13 0.84 0.72])
plot(0:2^n-1,dist)
axis([0 2^n-1 0 dm2])
xlabel('Ganzzahlinterpretation des Bitmusters','FontSize',fsz)
ylabel('Abstandsquadrat','FontSize',fsz)
title(sprintf('Das Bitmuster  %s  (dezimal %5d)\nliefert den kleinsten Abstand zum Input',...
      muster(1:end-1),bestk),'FontSize',fsz)
   
% Verteilung der Abstandsquadrate
fh2 = fullscreen('kumulative Verteilung','w',0,[2 2; 1 2]);
line(sdist(1),0,'Marker','.','MarkerSize',20,'LineStyle','none','Color','g')
line(sdist(2),2^-n,'Marker','.','MarkerSize',20,'LineStyle','none','Color','r')
line(sdist(3:end),(2:2^n-1)*2^-n,'Marker','o','MarkerSize',5,'LineStyle','none')
og = min([16383;dm2]);
xx = dm2*(0:399)/400;
line(xx,(1+erf((xx-dm)/sm))/2,'Color','r')
%line(xx,exp(-2*((xx-dm)/sm).^2),'Color','g')  % Verteilungsdichte
axis([0 dm2 0 1])
xlabel('Abstandsquadrat','FontSize',fsz)
title('kumulative Verteilung der Abstandsquadrate','FontSize',fsz)
legend('minimale Distanz','zweitkleinste Distanz','brige Distanzen','erwartete Verteilung',4)

% Abstandsquadrat ber dem Hammingabstand der Informationswrter
fh3 = fullscreen('V(Hammingabstand)','w',0,[2 2; 1 1]);
mnabs = inf;
for k=1:2^n-1
   hgabst = sum(u~=num2binvec(k-1,n));
   line(hgabst,dist(k),'Marker','*','LineStyle','none')
end
line([0 n],dist(1+binvec2num(u))*[1 1],'Color','r','LineStyle',':')
axis([0 n 0 dm2])
xlabel('Hammingabstand der Informationswrter','FontSize',fsz)
ylabel('Abstandsquadrat der Kodewrter','FontSize',fsz)
title({'Verteilung der Abstandsquadrate der Kodewrter';'ber dem Hammingabstand der Informationsmuster'},'FontSize',fsz)

