function demominrm(ftab)
% demominrm(ftab)
% Graphik zur Minimierung des Realisierungsaufwandes fr eine Reed-Muller-Form.
% ftab    : Vektor der Funktionstabelle mit 0 und 1 sowie -1 fr Don't- Care.
% Die Graphik ist nur sinnvoll fr Schaltfunktionen von 1 ... 6 Variablen.

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Default
% =======
if nargin<1  ftab =[1 -1  1  0  0  1  1 -1 -1  1 -1  0  0  1  0 -1]; end

% Prfung der Funktionstabelle
% ============================
if ~all(ftab==0 | ftab==1 | ftab==-1) error('demominrm: unzulssige Werte in ftab'); end 
if isempty(find(ftab==-1)) error('demominrm: ohne Don''t-Cares keine Minimierung!'); end
if length(ftab)>64 | length(ftab)<2
   error('demominrm ist nur sinnvoll fr Schaltfunktionen mit 1...6 Variablen!'); 
end

% Konstanten
% ==========
fsiz = 17;                         % FontSize
fsz  = 13;                         % FontSize
nn = length(ftab);                 % Lnge der Funktionstabelle
n  = round(log2(nn));              % Anzahl der Variablen
v  = 'ABCDEFGHJKLMNPQRSTUVWXYZ';   % Bezeichnungen der Unbekannten
variable = {'a';'b';'c';'d';'e';'f';'g';'h'}; % Variablennamen
ix = find(ftab<0);                 % Indices der Don't- Cares in ftab
m  = length(ix);                   % Anzahl der Don't-Cares
%px = [13 20 23 30 3*m+[32 40 3*n+40]]; % x-Positionen der Tabellen
switch n
case {1,2}
   ax = [0 100 -12 +25];
   tym  = -8;
   tyt  = 20;
case 3
   ax = [0 100 -30 +32];
   tym  = -20;
   tyt  = 27;
case 4
   ax = [0 100 -55 +50];
   tym  = -50;
   tyt  = 44;
case 5
   ax = [0 100 -200 +60];
   fsiz = 13;
   tym  = -190;
   tyt  = 48;
case 6
   ax = [0 100 -500 +150];
   fsiz = 13;
   tym  = -600;
   tyt  = 130;
end
px = [13 20 23 30 0.13*fsiz*m+[34 40 3*n+40]]; % x-Positionen der Tabellen

% Transformationsmatrix aufstellen
% ================================
k = nn;
A = 1;
Z = 0;
while k>1
   A = [A Z; A A];
   Z = [Z Z; Z Z];
   k = k/2;
end

% Minimierung
% ===========
[ftm,rm,sing,t0] = minrm(ftab);

% Ermittlung der Tabellen
% =======================
bb  = blanks(2*m-1);
for k=1:nn  B(k,:) = bb; end
rm0 = rem(A*(ftab'>0),2);          % Reed-Muller-Form mit Don't-Cares = 0
for k=1:nn  
   if rm0(k)  B(k,1) = '1'; else B(k,1) = '0'; end
end
for k=1:m
   iy = find(A(:,ix(k)));
   B(iy,2*k) = '+';   
   B(iy,2*k+1) = v(k);             % individuelle Transformation der Don't-Cares
end

sft = ftabout(ftab)';              % Funktionstabelle
sfu = sft;
sfu(ix) = v(1:length(ix));         % Funktionstabelle mit Unbekannten anstelle *

for k=1:nn  nul(k,:) = '   '; end
for k=1:length(t0) nul(t0(k),:) = '= 0'; end 

% Terme
% =====
trm{1} = '1';
for k=2:nn                         % alle Terme der Reed-Muller-Form
   t = '';
   bvk = num2binvec(k-1,n);        % bvk zeigt welche Variablem im Term vorkommen
   for j=1:n
      if bvk(n-j+1)                % wenn Variable vorkommt dann einfgen
         t = [t sprintf('%s*',variable{j})];
      end
   end
   trm{k} = t(1:end-1);
end

% Graphik
% =======
fh(1) = fullscreen('MinRM');
axes('Position',[0 0 1 1])
text(px(1),0,binmatout(num2binmat((0:nn-1)',n)),'FontName','FixedWidth','FontSize',fsiz,...
     'VerticalAlignment','baseline');
text(px(2),0,sft,'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(3),0,sfu,'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(4),0,B,'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(5),0,nul,'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(6),0,trm,'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(7),0,binmatout(rm'),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(1),0.07*nn+1,strcat(variable{n:-1:1}),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
text(px(1)+n/2+1,0.13*nn+1.5,'Variable','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(2)+1,0.13*nn+1.5,'Funktionstabelle mit Don''t-Cares','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(3)+1,0.13*nn+1.5,'Don''t-Cares als formale Unbekannte','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(4)+1,0.13*nn+1.5,'Reed-Muller-Koef. mit Unbekannten','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(5)+1,0.13*nn+1.5,'Nullsetzen von aufwendigen Termen','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(6)+1,0.13*nn+1.5,'Terme der Reed-Muller-Form','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(px(7)+1,0.13*nn+1.5,'Koeffizienten der minimierten R-M-Form','FontSize',fsz,'VerticalAlignment','baseline','Rotation',45);
text(7,tyt,'Minimierung der Reed-Muller-Form unter Ausnutzung von Don''t-Cares','FontSize',20,...
     'FontWeight','bold','Color','b')
text(3,tym,['Minimierte Reed-Muller-Form:   ' rm2expr(rm,variable)],'FontName','FixedWidth','FontSize',fsiz,...
     'FontWeight','bold','Color','r')
axis(ax); axis off

if n>5
   rmexpr = rm2expr(rm,variable);
   iz   = find(rmexpr=='+');
   izz  = iz(ceil(length(iz)/2)+2);
   rmx1 = rmexpr(1:izz-1);
   rmx2 = rmexpr(izz:end);
   set(fh(1),'Name','MinRM...')
   fh(2) = fullscreen('...MinRM');
   axes('Position',[0 0 1 1])
   text(px(1),0,binmatout(num2binmat((nn/2:nn-1)',n)),'FontName','FixedWidth','FontSize',fsiz,...
        'VerticalAlignment','baseline');
   text(px(2),0,sft(nn/2+1:nn,:),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(px(3),0,sfu(nn/2+1:nn,:),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(px(4),0,B(nn/2+1:nn,:),  'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(px(5),0,nul(nn/2+1:nn,:),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(px(6),0,{trm{nn/2+1:nn}},'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(px(7),0,binmatout(rm(nn/2+1:nn)'),'FontName','FixedWidth','FontSize',fsiz,'VerticalAlignment','baseline');
   text(1,-430,sprintf('Minimierte Reed-Muller-Form:\n\n%s\n\n%s',rmx1,rmx2),'FontName','FixedWidth','FontSize',8,...
        'FontWeight','bold','Color','r')
   axis([0 100 -500 50]); axis off
end
