function yy = demoprediction(n,m,g)
% yy = demoprediction(n,m,g)
% Audiodatenkompression durch Prdiktion mit n Bits Genauigkeit und m 
% Bits fr die Differenzen.  g ist der Grad des Prdiktionspolynoms.
% Als Audiosignal wird ein Chirp generiert.
% Beispiele ohne Overload:  demoprediction(8,4,1);  demoprediction(8,3,2);
%   demoprediction(10,4,2); demoprediction(10,4,3); demoprediction(20,7,6);
% Beispiele mit Overload:   demoprediction(10,5,1); demoprediction(10,3,2);
%   demoprediction(8,5,0);
% Overloading ist sehr gut in der Vergrsserung des Audiosignals zu sehen.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Konstanten
% ==========
fs = 44100;                                   % Abtastrate wie CD
f0 = 0;                                       % Anfangsfrequenz des Chirps
f1 = 1000;                                    % Chirpfrequenz nach 0.2 s

% Defaults fr Parameter
% ======================
if nargin<1  n = 10;  end
if nargin<2  m = 6;   end
if nargin<3  g = 0;   end

% Audiodaten erzeugen: Chirp
% ==========================
t  = (0:fs/3)/fs;                             % Zeitbereich 1/3 Sekunde
x  = chirp(t,f0,0.2,f1);                      % Chirp
xq = round((x - min(x))/(max(x) - min(x))*(2^n - 1)); % Quantisierung in n Bit

% Deltamodulation
% ===============
dx = xq(1:g+1);                               % erste g+1 Samples direkt
xx = dx;
for k=1:length(xq)-g-1
%   p = polyfit(0:g,xx(k:k+g),g);              % Polynomfit in letzte g+1 Punkte
%   rpv = round(polyval(p,g+1));               % Prdiktion fr nchsten Punkt
   rpv = round(lagrangeinterpol(0:g,xx(k:k+g),g+1));
   d = xq(k+g+1) - rpv;                       % Differenz (Soll - Prdiktion)
   dx(k+g+1) = min(2^m-1,max(0,d + 2^(m-1))); % Reprsentation in m Bit
   xr = dx(k+g+1) - 2^(m-1) + rpv;            % Simulation der Rekonstruktion
   xx(k+g+1) = min(2^n-1,max(0,xr));          % Reprsentation in n Bit
end

% Rekonstruktion
% ==============
yy = dx(1:g+1);                               % erste g+1 Samples direkt
for k=1:length(dx)-g-1          
%   p  = polyfit(0:g,yy(k:k+g),g);             % Polynomfit in letzte g+1 Punkte
%   yr = dx(k+g+1) - 2^(m-1) + round(polyval(p,g+1)); % Rekonstruktion
   yr = dx(k+g+1) - 2^(m-1) + round(lagrangeinterpol(0:g,xx(k:k+g),g+1));
   yy(k+g+1) = min(2^n-1,max(0,yr));          % Reprsentation in n Bit
end

% Graphik
% =======
fh1 = fullscreen('Audio','w',3,[3,1;1,1]);
axes('Position',[0 0 1 1])
line(1:length(xq),xq,'Marker','.','Color','b') % Inputsignal
line(1:length(yy),yy,'Marker','.','Color','r') % Outputsignal
line(1:length(dx),dx*2^(n-m),'Marker','.','Color','g') % bertragene Differenzen
axis([666 length(xq) -2^(n-2) 2^n+2^(n-2)])
axis off
legend('Originaldaten','Rekonstruktion','bertragene Daten')

fh2 = fullscreen('Fehler','w',0,[3,1;2,1]);
axes('Position',[0 0 1 1])
plot(yy-xq)                                   % Differenz zwischen Input- und Outputsignal
axis([666 length(xq) -10 +10])
axis off

fh31 = fullscreen('Demo 1','w',0,[3,3;3,1]);
k = round(length(xx)/8);                      % Beispiel nahe am Anfang der Sequenz
p = polyfit(0:g,xx(k:k+g),g);                 % Polynomfit in letzte g+1 Punkte
rpv = round(polyval(p,g+1));                  % Prdiktion fr nchsten Punkt
d = xq(k+g+1) - rpv;                          % Differenz (Soll - Prdiktion)
dxkg1 = min(2^m-1,max(0,d + 2^(m-1)));        % Reprsentation in m Bit
xr = dxkg1 - 2^(m-1) + rpv;                   % Simulation der Rekonstruktion
xxkg1 = min(2^n-1,max(0,xr));                 % Reprsentation in n Bit
line(k:k+g+1,xq(k:k+g+1),'Marker','.','Color','b','LineStyle','none','MarkerSize',30) % Inputsignal
line(k:k+g,xx(k:k+g),'Marker','.','Color','r','LineStyle','none','MarkerSize',20) % Outputsignal
line(k-0.5:0.01:k+g+1.5,polyval(p,-0.5:0.01:g+1.5),'Color','r','LineWidth',1.5) % Polynomfit
line(k+g+1,rpv,'Marker','x','Color','r','LineStyle','none','MarkerSize',12,'LineWidth',2)  % Prdiktion
line(k+g+1,xxkg1,'Marker','o','Color','r','LineStyle','none','MarkerSize',10,'LineWidth',2) % Rekonstruierter Wert
legend('Inputdaten','vorliegende Rekonstruktion','Polynomfit','Prdiktion','nchste Rekonstruktion')

fh32 = fullscreen('Demo 2','w',0,[3,3;3,2]);
k = round(length(xx)-length(xx)/8);           % Beispiel nahe am Ende der Sequenz
p = polyfit(0:g,xx(k:k+g),g);                 % Polynomfit in letzte g+1 Punkte
rpv = round(polyval(p,g+1));                  % Prdiktion fr nchsten Punkt
d = xq(k+g+1) - rpv;                          % Differenz (Soll - Prdiktion)
dxkg1 = min(2^m-1,max(0,d + 2^(m-1)));        % Reprsentation in m Bit
xr = dxkg1 - 2^(m-1) + rpv;                   % Simulation der Rekonstruktion
xxkg1 = min(2^n-1,max(0,xr));                 % Reprsentation in n Bit
line(k:k+g+1,xq(k:k+g+1),'Marker','.','Color','b','LineStyle','none','MarkerSize',30) % Inputsignal
line(k:k+g,xx(k:k+g),'Marker','.','Color','r','LineStyle','none','MarkerSize',20) % Outputsignal
line(k-0.5:0.01:k+g+1.5,polyval(p,-0.5:0.01:g+1.5),'Color','r','LineWidth',1.5) % Polynomfit
line(k+g+1,rpv,'Marker','x','Color','r','LineStyle','none','MarkerSize',12,'LineWidth',2)  % Prdiktion
line(k+g+1,xxkg1,'Marker','o','Color','r','LineStyle','none','MarkerSize',10,'LineWidth',2) % Rekonstruierter Wert
legend('Inputdaten','vorliegende Rekonstruktion','Polynomfit','Prdiktion','nchste Rekonstruktion')

fh33 = fullscreen('Verteilung','w',0,[3,3;3,3]);
hist(dx(n+1:end),50)                          % Verteilung der Fehler der Prdiktion
title('Verteilung der Fehler der Prdiktion (bertragene Daten)')

% Audio
% =====
soundsc(x,fs); pause(0.5)                     % Input
ys = yy - mean(yy);
ys = ys/max(abs(ys));                         % skaliertes Outputsignal
soundsc(ys,fs)                                % Output
