function drawfsm(stab,sname,z0)
% drawfsm(stab,sname,z0)
% Zustandsgraph fr ein Schaltwerk mit der 
% bergangstabelle stab.
%
% stab  :  m*n-Matrix, die in der k-ten Zeile die Indices der
%          Folgezustnde fr den Zustand mit dem Index k enthlt
%          und zwar in der Spalte j den zur Eingangsbelegung mit 
%          dem Index j gehrigen Folgezustand.
%          Ist der Index <1, so wird keine Kante gezeichnet.  
% sname :  StructureArray mit den Strings der Zustandsnamen.
%          Die Namen sollten hchstens 3 Buchstaben haben.
% z0    :  Index des Anfangszustands (Default 1)
%
% Beispiel:  drawfsm([1 2; 3 4; 2 3; 1 4],{'RR';'Go';'Z1';'Z2'})

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  stab = [1 2; 3 4; 2 3; 1 4]; end
if nargin<2  
   sname = {'A';'B';'C';'D';'E';'F';'G';'H';'I';'J';'K';'L';'M';...
            'N';'O';'P';'Q';'R';'S';'T';'U';'V';'W';'X';'Y';'Z'};
   k = 1;   
   while length(sname)<length(stab)
      sname = cat(1,sname,strcat(sname{k},sname));
      k = k + 1;
   end
end
if nargin<3  z0 = 1; end

% Konstanten
% ==========
lw = 1.5;                    % LineWidth
nc = 0.7*[1 1 1];            % Knoten-Farbe
sc = 'g';                    % Farbe selektierter Knoten
nl = 'k';                    % Kanten-Farbe
sl = 'r';                    % Farbe selektierter Kanten
[ns,nx] = size(stab);        % Anzahl von Knoten und mglichen Inputs
r  = 1/ns;                   % Knotenradius
sf = 0.25/ns;                % Buchstabenabstand in Kantenbeschriftung
fsz= min(20,160*r);          % Fontsize fr Zustandsbeschriftung

% Hilfetext
% =========
helptext = {'Hilfe zum Zustandsdiagramm';
            '=====================';
            '';
            'Whlen des Eingabeindex:';
            '   Tasten 1, 2, ... 9';
            '';
            'Fortschalten in Folgezustand:';
            '   Taste c (clock) oder t (Takt)';
            '   oder n (next), auch C,T,N'};

% Schlingenprototyp
% =================
u  = pi*(2:18)/10;           % Polygonwinkel 
sx = r*sin(u);               % x-Koordinaten der Polygonsttzstellen
sy = 1 + 2*r - 1.5*r*cos(u); % y-Koordinaten der Polygonsttzstellen
tx = 0;                      % x-Koordinate des Textes
ty = 1 + 2*r;                % y-Koordinate des Textes

% Fenster
% =======
fh = fullscreen('Zustandsdiagramm','w',0,[0 0.75 0 1]);
axes('Position',[0 0 1 1])

% Zustandsknoten
% ==============
a  = (0:2*pi/ns:1.99*pi)';   % Winkelpositionen der Knoten
cc = [sin(a) cos(a)];        % Positionen der Zustandsknoten
for k=1:ns
   sh(k) = drawccir(cc(k,:),r,nc,sname{k},fsz);
end

% bergnge
% =========
for k=1:ns
   for j=1:nx
      fs = stab(k,j);        % Folgezustand
      dx = sf*sum(stab(k,1:j)==fs);
      if fs==k               % Schlinge
         A  = [cc(k,[2 1]); -cc(k,1) cc(k,2)];
         xy = A*[sx;sy];
         ddx= sf*sum(stab(k,1:nx)==fs)/2;
         txy= A*[tx;ty];
         px = [xy(1,1) xy(1,2)+0.3*(xy(2,2)-xy(2,1))*[-1 +1]];
         py = [xy(2,1) xy(2,2)-0.3*(xy(1,2)-xy(1,1))*[-1 +1]];
         lh{k,j} = [line(xy(1,:),xy(2,:),'Color',nl,'LineWidth',lw),...
                    patch(px,py,nl),...
                    text(txy(1)+dx-ddx,txy(2),num2str(j),'FontName','FixedWidth',...
                         'VerticalAlignment','middle','HorizontalAlignment','center','FontUnits','normalized')];
      elseif fs>0            % Kante
         dd = cc(fs,:) - cc(k,:);
         ll = sqrt(dd*dd');
         if prod(dd)>0
            xy = cc(k,:) + 1.8*r*dd/ll;
            thnd = text(xy(1)+dx-0.02,xy(2),num2str(j),'FontName','FixedWidth',...
                        'VerticalAlignment','top','HorizontalAlignment','left','FontUnits','normalized');
         else
            xy = cc(k,:) + 1.8*r*dd/ll;
            thnd = text(xy(1)-dx+0.03,xy(2),num2str(j),'FontName','FixedWidth',...
                        'VerticalAlignment','top','HorizontalAlignment','right','FontUnits','normalized');
         end
         lh{k,j} = [drawarr(cc(k,:),cc(fs,:),nl,lw,r/3,r,r),thnd];
      end
   end
end

axis((1+6/ns)*[-1 +1 -1 +1]); axis off

% Interaktion
% ===========
fud.helptext = helptext;
fud.nodes    = sh;
fud.edges    = lh;
fud.stab     = stab;
fud.currs    = z0;
fud.currx    = 1;
fud.fig      = [];
fud.figfcn   = [];
set(fh,'UserData',fud,'KeyPressFcn','drawfsmkpf');
set(fud.nodes(fud.currs),'FaceColor',sc)
set(findobj(fud.edges{fud.currs,fud.currx},'Type','patch'),'FaceColor',sl)
set(findobj(fud.edges{fud.currs,fud.currx},'Type','line'),'Color',sl,'LineWidth',3)
