function kvdiagram(sfcn,variable,aw,loop,currloop)
% kvdiagram(sfcn,variable,aw,loop,currloop)
% Interaktive Graphik des KV-Diagramms fr die durch sfcn 
% definierte Schaltfunktion mit den Variablen variable. 
%
% sfcn kann auf folgende Weise gegeben werden:
% sfcn = natrliche Zahl. In diesem Fall wird ein KV-Diagramm 
%        gezeichnet mit den Indices der Felder anstelle der 
%        Funktionswerte
% sfcn = Vektor der Lnge 2^n , der die Funktionstabelle enthlt
%        (0 bzw. 1 fr festgelegte Werte; <0 fr Don't Care
% sfcn = String der Lnge 2^n mit den Bits der Funktionstabelle 
%        ('0' bzw. '1' bzw. '*')
% sfcn = String eines Ausdrucks in den Variablen variable
%
% variable :  Cell array mit den Variablen als Strings
%             Die Variablen drfen nicht binexp oder variable
%             heissen. Auch die Schlsselwrter von MATLAB 
%             wie for, while, end, case etc. sind nicht erlaubt.
%             Dieser Parameter ist nur erforderlich im Fall,
%             dass sfcn als Ausdruck in diesen Variablen vorliegt.
%             Default ist  {'a'; 'b'; 'c'; 'd'}
%
% aw = [na, no] :  Aufwand an Und und Oder-Gliedern
% 
% Zur Darstellung schon gewhlter Schleifen dienen:
% 
% loop     :  CellArray mit den Listen der Schleifenteilnehmer
% currloop :  fokussierte Schleife
% Diese Daten knnen nach dem Whlen der Schleifen mit 
% get(gcf,'UserData') erfragt werden.
%
% Beispiele:
% kvdiagram(5)
% kvdiagram(randb(16),{'x_1';'x_2';'x_3';'x_4'})
% kvdiagram('(u&~v)|~w',{'u';'v';'w'})
%
% Interaktion
% ===========
% Disjunktive und konjunktive Form:
% Nach dem Aufruf von kvdiagram kann mit der Taste K oder k in die 
% konjugierte Form gewechselt werden.  Ein Wechsel zurck ist mit 
% der Taste D oder d mglich, wenn alle evtl. schon geklickten 
% Schleifen weggeklickt sind.
%
% Schleifen:
% Felder werden durch Anklicken hinzugefgt bzw. entfernt. 
% Eine neue Schleife beginnt man mit der Taste + .  Mit den 
% Tasten + und - kann man von Schleife zu Schleife wechseln 
% und dann in der angewhlten Schleife Korrekturen machen.
%
% Terme:
% Mit Klick auf einen der Schleifenbuttons in der rechten 
% Schleifenspalte wird der zugehrige Term berechnet.
%
% Minimierter Ausdruck:
% Mit der Taste r wird die erreichte dijunktive oder konjunktive 
% Form in MATLAB-Syntax im Kommandofenster ausgegeben.

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1  sfcn = 4; end  
if nargin<2
   if length(sfcn)==1  kk = sfcn; else kk = round(log2(length(sfcn))); end
   v  = 'abcdefghijklmnopqrstuvwxyz';
   for k=1:kk  variable{k} = v(k); end 
end
if nargin<3  aw = [0 0]; end

% Konstanten
% ==========
[fname,fwgt,sz,pres] = getfont;            % Font
if sz>12
   scrs = get(0,'ScreenSize');
   fsz  = round(20/1024*scrs(3));          % FontSize
   fws  = fsz + 5;
else
   fsz  = sz;
   fws  = sz;
end
%fws = 20;                                  % FixedWidthFontSize
%fsz = 17;                                  % FontSize
col = color(1:64,0.7,3);
col = col(1+rem(1:4:2000,59),:);           % Feld-Farben
ss  = get(0,'ScreenSize');                 % Bildschirmauflsung

% Hifetext
% ========
helptext = {'Hilfe zu kvdiagram';
            '------------------------';
            '';
            'Editieren der Funktionswerte:';
            '   Nach Bedarf mit den Tasten 0 1 * einen Wert whlen';
            '   und nun den Wert mit der Maus in die Felder klicken.';
            '';
            'Whlen der konjunktiven bzw. disjunktiven Form:';
            '   Taste k bzw d tippen. Ein Wechsel ist nur mglich,';
            '   wenn keine Schleifen existieren.';
            '';
            'Bilden einer Schleife:';
            '   Mit der Maus gewhlte Felder anklicken.';
            '   Wiederholtes Klicken entfernt das Feld.';
            '';
            'Neue Schleife beginnen:';
            '   Mit der Taste + wird die nchste Schleife gewhlt,';
            '   mit der Taste - die vorige.';
            '';
            'Primimplikant anzeigen:';
            '   Klicken auf den farbigen Schleifenbutton (Spalte rechts).';
            '';
            'Minimierten Ausdruck in Matlab-Syntax und Aufwand ausgeben:';
            '   Taste r tippen.';
            '';
            'Entfernen des Bildes mit der Taste ENTF';
            '';
            'Entfernen aller Bilder mit der Taste ESC'};

% Berechnung der Funktionstabelle in f
% ====================================
if ischar(sfcn)
   if nargin==1                            % Definition als Ausdruck ?
      n = round(log(length(sfcn))/log(2)); % Anzahl der Variablen
      f = 3*ones(1,2^n);                   % Vorbesetzung mit Fehlercode
      f(findstr('0',sfcn)) = 0;            % Nullen in f setzen
      f(findstr('1',sfcn)) = 1;            % Einsen in f setzen
      f(findstr('*',sfcn)) =-1;            % Don'Care in f setzen
   else                                    % Definition als Funktionstabelle in String
      f = binexp2bintab(sfcn,variable);    % Berechnung der Funktionstabelle   
   end
elseif length(sfcn)==1
   f = 1:2^sfcn;                           % Indices statt Funktionstabelle
else
   f = sfcn;                               % Funktionstabelle ist direkt gegeben
end

% KV-Diagramm zeichnen
% ====================
switch length(variable)
case {1,2},  fp1 = [3 130 580 200];  fp2 = [3 30 580 100];
case {3,4},  fp1 = [3 180 580 315];  fp2 = [3 30 580 150];
case {5,6},  fp1 = [3 280 600 470];  fp2 = [3 30 600 260];
otherwise
   fp1 = [0 ss(4)/3 ss(3) ss(4)/3*2]; fp2 = [0 0 ss(3) ss(4)/3];
end
fh = fullscreen('KV-Diagramm','w',0,[0 1 0.3 1]);
if ~pres  set(fh,'Position',fp1); end
ax1= axes('Position',[0 0 ss(4)/ss(3) 1]);
axis square
p  = log2(length(f));
ph = drawkv([0 0 1 1/(1+rem(p,2))],f,variable,1:2^p);  % rekursiv zeichnen
if length(f)>8  s = 0.97; else s = 1.5; end
axis(s*[-1 +1 -1 +1]); axis off

% Minterm-Fenster
% ===============
fh2 = fullscreen('Minterme','w',0,[0 1 0 0.3]);
if ~pres  set(fh2,'Position',fp2); end
ax3 = axes('Position',[0 0 1 1]);
ph2 = patch([0 100 100 0],[-0.2 -0.2 2 2],col(1,:));
xa  = 10;
text(xa+1,1.5,'Feld-Nr.','FontSize',fsz,'FontUnits','normalized')
text(1,0.5,'Variable','FontSize',fsz,'FontUnits','normalized')
line([0 xa; 100 xa],[-0.2 -length(variable)-1; -0.2 2],'Color','k')
for k=1:length(variable)
   text(1,-k,variable(k),'FontSize',fsz,'FontUnits','normalized')
   mth(k) = text(xa+1,-k,'','FontSize',fws,'FontName','FixedWidth','FontUnits','normalized');
end
mth(length(variable)+1) = text(xa+1,0.5,'','FontSize',fws,'FontName','FixedWidth','FontUnits','normalized');
axis([0 100 -length(variable)-0.5 2]); axis off

% Interaktion ermglichen
% =======================
figure(fh);
fud.variable = variable;
fud.tab = f;
fud.currloop = 1;
if nargin>3
   fud.loop = loop;
else
   fud.loop = {[]};
end
fud.ph = ph;
ax2 = axes('Position',[ss(4)/ss(3) 0 1-ss(4)/ss(3) 1]);
fud.ax = [ax1 ax2];
text(0,0,'Schleife  /  Term','FontSize',15,'FontUnits','normalized')
ns = min(32,2^p);
dx = min(80/ns,10);
for k=1:ns
   lph(k) = patch(dx/2+[0 dx dx 0],-k-[0 0 1 1],col(k,:));  % Term-Feld
   lth(k) = text(2*dx,-k-0.5,'','FontSize',fsz,'FontUnits','normalized'); % Term-Text
   set(lph(k),'UserData',k,'ButtonDownFcn',@kvdiagrambdf)
end
set(lph(length(fud.loop)+1:end),'Visible','off')
axis([0 40 -ns-3 1]); axis off

fud.helptext = helptext;
fud.lph = lph;
fud.lth = lth;
fud.mth = mth;
fud.dmf = 1;
fud.ph2 = ph2;
fud.aw  = aw;
set(fh,'UserData',fud,'KeyPressFcn',@kvdiagramkpf)

if nargin>3
   fud.loop = loop;
   for k=1:length(loop)
      fud = get(fh,'UserData');
      fud.currloop = k;
      set(fh,'UserData',fud,'KeyPressFcn',@kvdiagramkpf)
      kvdiagrambdf
   end
   if nargin>4  
      fud = get(fh,'UserData');
      fud.currloop = currloop;
      set(fh,'UserData',fud,'KeyPressFcn',@kvdiagramkpf)
      kvdiagrambdf
   end
end

%--------------------------------------------------------------------------

function ph = drawkv(pos,fkv,variable,in)
% drawkv ist eine rekursive Hilfsfunktion zum Zeichnen eines KV-Diagramms
%fsiz = 3 + 80/length(variable);
fsiz = 15;
a = 0.7;
n = length(fkv);
p = round(log2(n));
m = length(variable);
k = m - p;
if n==1
   if fkv<0  s = '*'; else s = num2str(fkv); end
   [ph,th] = drawcsq(pos(1:2),2*pos(3),'w',s,fsiz-3);
   ptud.index = in;
   ptud.ph = ph;
   ptud.th = th;
   ptud.ls = [];
   set([ph th],'ButtonDownFcn',@kvdiagrambdf,'UserData',ptud)
else
   if rem(p,2)
      sx = pos(3)/2;
      sy = pos(4);
      if p==3  
         ix = [(5*n/8+1:6*n/8),(4*n/8+1:5*n/8),(7*n/8+1:n),(6*n/8+1:7*n/8)]; 
      else 
         ix = (n/2+1:n); 
      end
      ph = [drawkv([pos(1)-sx pos(2) sx sy],fkv(1:n/2),variable,in(1:n/2)), ...
            drawkv([pos(1)+sx pos(2) sx sy],fkv(ix),variable,in(ix))];
      ph = ph([1:n/2 ix]);
   else
      sx = pos(3);
      sy = pos(4)/2;
      if p>3  sx = a*sx; sy = a*sy; end
      if p==4  
         ix = [(5*n/8+1:6*n/8),(4*n/8+1:5*n/8),(7*n/8+1:n),(6*n/8+1:7*n/8)]; 
      else 
         ix = (n/2+1:n); 
      end
      ph = [drawkv([pos(1) pos(2)+sy sx sy],fkv(1:n/2),variable,in(1:n/2)), ...
            drawkv([pos(1) pos(2)-sy sx sy],fkv(ix),variable,in(ix))];
      ph = ph([1:n/2 ix]);
   end
   if (p==m)|(rem(p,4)==0)|((rem(p,2)==0)&p>4)
      u = 1.1;
      v = 1.12;
      if rem(p,2)
         line(pos(1)+[0 2*sx],pos(2)+u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1)+sx,pos(2)+v*sy,variable(k+1),'HorizontalAlignment','center', ...
              'VerticalAlignment','bottom','FontSize',fsiz,'FontUnits','normalized')
      else
         line(pos(1)-u*sx+[0 0],pos(2)-[2*sy 0],'Color','k','LineWidth',1.5)
         text(pos(1)-v*sx,pos(2)-sy,variable(k+1),'HorizontalAlignment','right', ...
              'VerticalAlignment','middle','FontSize',fsiz,'FontUnits','normalized')
         line(pos(1)+[0 sx],pos(2)+2*u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1)+sx/2,pos(2)+2*v*sy,variable(k+2),'HorizontalAlignment','center', ...
              'VerticalAlignment','bottom','FontSize',fsiz,'FontUnits','normalized') 
      end
      if p==4
         line(pos(1)+0.5*sx*[-1 +1],pos(2)-2*u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1),pos(2)-2*v*sy,variable(k+4),'HorizontalAlignment','center', ...
              'VerticalAlignment','top','FontSize',fsiz,'FontUnits','normalized')
         line(pos(1)+u*sx+[0 0],pos(2)-sy*[-1 +1],'Color','k','LineWidth',1.5)
         text(pos(1)+v*sx,pos(2),variable(k+3),'HorizontalAlignment','left', ...
              'VerticalAlignment','middle','FontSize',fsiz,'FontUnits','normalized')         
      end
      if p==3
         line(pos(1)+sx*[-1 +1],pos(2)-u*sy*[1 1],'Color','k','LineWidth',1.5)
         text(pos(1),pos(2)-v*sy,variable(k+3),'HorizontalAlignment','center', ...
              'VerticalAlignment','top','FontSize',fsiz,'FontUnits','normalized')
         line(pos(1)-2*u*sx+[0 0],pos(2)-[sy 0],'Color','k','LineWidth',1.5)
         text(pos(1)-2*v*sx,pos(2)-sy/2,variable(k+2),'HorizontalAlignment','right', ...
              'VerticalAlignment','middle','FontSize',fsiz,'FontUnits','normalized')
      end
   end
end

%-------------------------------------------------------------------------------

function kvdiagramkpf(obj,eventdata)
% kvdiagramkpf
% KeyPressFcn fr kvdiagram

% global KVMINEXP KVMINCOST;

col = color(1:64,0.7,3);
col = col(1+rem(1:4:2000,59),:);
cfg = gcf;
fud = get(cfg,'UserData');
cc  = get(cfg,'CurrentCharacter');
switch cc
case '0'
   fud.currloop = 0;
   fud.cc = '0';
   nl = ones(1,128)*nan; 
   nl([53:60 68:77 83 84 93 94 98 99 110 111 114 115 126 127]) = 1; 
   nll = reshape([nl reverse(nl)],16,16); 
   set(gcf,'Pointer','custom','PointerShapeCData',nll,'PointerShapeHotSpot',[8 8])
case '1'
   fud.currloop = 0;
   fud.cc = '1';
   nl = ones(1,256)*nan; 
   nl([85 100 101 115 116 130:143 146:159]) = 1; 
   nll = reshape(nl,16,16); 
   set(gcf,'Pointer','custom','PointerShapeCData',nll,'PointerShapeHotSpot',[8 8])
case '*'
   fud.currloop = 0;
   fud.cc = '*';
   nl = ones(1,128)*nan; 
   nl([37 44 53 54 59 60 70 75 87 90 104 105 114:127]) = 1; 
   nll = reshape([nl reverse(nl)],16,16); 
   set(gcf,'Pointer','custom','PointerShapeCData',nll,'PointerShapeHotSpot',[8 8])
case '+'
   if ~isempty(fud.loop{fud.currloop})
      fud.currloop = fud.currloop + 1;
   end
   if fud.currloop>length(fud.loop)  
      fud.loop{fud.currloop} = [];
      set(fud.lph(fud.currloop),'Visible','off')
   end
   set([fud.ph(fud.loop{fud.currloop}),fud.ph2],'FaceColor',col(fud.currloop,:))
   set(fud.lph(1:fud.currloop),'Visible','on')
case '-'
   fud.currloop = max(1,fud.currloop - 1);
   set([fud.ph(fud.loop{fud.currloop}),fud.ph2],'FaceColor',col(fud.currloop,:))
case 'r'
   n = round(log2(length(fud.tab)));
   if fud.dmf  
      dkop = '&';
      dkc  = ') | (';
      dks1 = '';
      dks0 = '~';
   else
      dkop = '|';
      dkc  = ') & (';
      dks1 = '~';
      dks0 = '';
   end
   r = '(';
   for k=1:length(fud.loop)
      if ~isempty(fud.loop{k})
         x = num2binmat(fud.loop{k}'-1,n);
         for j=1:n
            if all(x(:,j))   r = [r dks1 fud.variable{j} dkop]; end
            if all(~x(:,j))  r = [r dks0 fud.variable{j} dkop]; end
         end
         r = [r(1:end-1) dkc];
      end
   end
   mexpr = r(1:end-4);
   cost  = aufwand(mexpr);
   btb   = binexp2bintab3(mexpr,fud.variable);
   irel  = find(fud.tab>=0);
   if ~all(btb(irel)==fud.tab(irel))
      result2fig('wenigstens eine Schleife ist fehlerhaft')
   else
%       KVMINEXP  = mexpr;
%       KVMINCOST = cost;
      if length(mexpr)<87
         Resultat{1} = ['Minimalform: ' mexpr];
         Resultat{2} = ['Kosten:      ' num2str(cost)];
         result2fig(Resultat)
      else
         Kosten = cost;
         result2fig(Kosten)
         Minimalform = mexpr;
         result2fig(Minimalform)
      end
      aw = fud.aw;
      if any(aw)
         checkkvmin(mexpr,fud.tab,fud.variable,aw(1),aw(2))
      end
   end
case {'D','d'}
   ll = [];
   if fud.dmf==0
      for j=1:length(fud.loop)
         ll(j) = length(fud.loop{j});
      end
   end
   if any(ll)
      sound(cos(pi/4*(1:1000)),8000)
      return
   else
      fud.dmf = 1;
      set(gcf,'Pointer','arrow')
   end
   fud.currloop = 1;
case {'K','k'}
   ll = [];
   if fud.dmf==1
      for j=1:length(fud.loop)
         ll(j) = length(fud.loop{j});
      end
   end
   if any(ll)
      sound(cos(pi/4*(1:1000)),8000)
      return
   else
      fud.dmf = 0;
      set(gcf,'Pointer','arrow')
   end
   fud.currloop = 1;
case {'p','P'}
%   Hier knnte die berprfung erfolgen !
case {'h','H'}
   fsiz = 13;
   fud = get(cfg,'UserData');
   nz = length(fud.helptext);
   fh = fullscreen('Hilfe','w',0,[0 0.6 0 1]);
   set(fh,'ButtonDownFcn','delete(gcf)')
   axes('Position',[0.1 0.1 0.8 0.8])
   axis([0 1 -nz 2])
   axis off
   text(0,-nz/2,fud.helptext,'FontSize',fsiz)
end
set(cfg,'UserData',fud)

if double(cc)==27  msvdelfig;   end   % delete all
if double(cc)==127 delete(gcf); end   % delete

%-------------------------------------------------------------------------------

function kvdiagrambdf(obj,eventdata)
% kvdiagrambdf
% ButtonDownFcn fr kvdiagram

col  = color(1:64,0.7,3);
col  = col(1+rem(1:4:2000,59),:);
cfg  = gcf;
cbo  = gcbo;
fud  = get(cfg,'UserData');
ptud = get(cbo,'UserData');
if isempty(ptud)  ptud = fud.currloop; end

if gca==fud.ax(2)                             % Klick auf Term-Feld
   fud.currloop = ptud;
   k = fud.currloop;
   loopk = fud.loop{k};
   mm = length(loopk);
   if mm>0
      lm = log2(mm);
   else
      lm = 0;
   end
   set([fud.ph(loopk),fud.ph2],'FaceColor',col(k,:))
   if ~isempty(loopk) & 2^round(lm)==mm
      n = length(fud.variable);
      x = num2binmat(loopk'-1,n);
      [m,dummy] = size(x);
      mtnr = '';
      for i=1:length(loopk)
         mtnr = [mtnr sprintf('%4d',loopk(i))];
      end
      set(fud.mth(n+1),'String',mtnr)
      if fud.dmf
         dks1 = '\cap';
         dks0 = '''\cap';
      else
         dks1 = '''\cup';
         dks0 = '\cup';
      end
      r  = [];
      ix = [];
      for j=1:n
        mtrm = '';
        for i=1:length(loopk)
           mtrm = [mtrm sprintf('%4d',x(i,j))];
        end
        set(fud.mth(j),'String',mtrm)
         if all(x(:,j))   
            set(fud.mth(j),'Color','r')
            r  = [r fud.variable{j} dks1 ];
         elseif all(~x(:,j))  
            set(fud.mth(j),'Color','r')
            r  = [r fud.variable{j} dks0];
         else
            ix = [ix j];
            set(fud.mth(j),'Color',[0 0.8 0])
         end
      end
      if all(sort(binmat2num(x(:,ix)))==(0:m-1))
         r = r(1:end-4);
%      if ~ismember(length(loopk),2.^(0:n))
      else
         r = '';
      end
   else
      r = '';
   end
   set(fud.lth(k),'String',sprintf('%s',r))
else                                          % Klick in KV-Diagramm-Feld
   if fud.currloop==0                         % Funktionswerte setzen
      switch fud.cc
         case '0', fud.tab(ptud.index) = 0;
         case '1', fud.tab(ptud.index) = 1;
         case '*', fud.tab(ptud.index) = -1;
      end
      set(ptud.th,'string',fud.cc)
   else                                       % Schleifen bilden
      loop = fud.loop{fud.currloop};
      if ismember(ptud.index,loop)            % Term aus Schleife entfernen
         loop = setdiff(loop,ptud.index);
         ptud.ls = setdiff(ptud.ls,fud.currloop);
         if length(ptud.ls)<1
            lc = [1 1 1];
         else
            lc = col(ptud.ls(end),:);
         end
         set(ptud.ph,'FaceColor',lc)
         fud.loop{fud.currloop} = loop;
      else                                    % Term in Schleife einfgen
         if fud.tab(ptud.index)~=-1 & fud.tab(ptud.index)~=fud.dmf
            sound(cos(pi/4*(1:1000)),8000)
            return
         end
         set(ptud.ph,'FaceColor',col(fud.currloop,:))
         fud.loop{fud.currloop} = [loop ptud.index];
         ptud.ls = [ptud.ls fud.currloop];
      end 
      k = fud.currloop;
      if isempty(fud.loop{k})
            set(fud.mth,'String','')
      else
         n = length(fud.variable);
         x = num2binmat(fud.loop{k}'-1,n);
         mtnr = '';
         for i=1:length(fud.loop{k})
            mtnr = [mtnr sprintf('%4d',fud.loop{k}(i))];
         end
        set(fud.mth(n+1),'String',mtnr)
        for j=1:n
           mtrm = '';
           for i=1:length(fud.loop{k})
              mtrm = [mtrm sprintf('%4d',x(i,j))];
           end
           set(fud.mth(j),'String',mtrm)
           if all(x(:,j))   
              set(fud.mth(j),'Color','r')
           elseif all(~x(:,j))  
              set(fud.mth(j),'Color','r')
           else
              set(fud.mth(j),'Color',[0 0.8 0])
           end
        end
      end
      set(fud.lth(k),'String','')
   end
   set(cbo,'UserData',ptud)
end
set(cfg,'UserData',fud)

%-------------------------------------------------------------------------------

function b = checkkvmin(binexp,ftab,variable,na,no)
% b = checkkvmin(binexp,ftab,variable,na,no)
% berprfung der Minimierung mit dem KV-Diagramm:
% binexp   :  String mit dem zu prfenden Ausdruck in MATLAB-Syntax
% ftab     :  Funktionstabelle der zu realisierenden Schaltfunktion
% variable :  Cell array mit den Variablen als Strings
% na       :  Anzahl der UND-Verknpfungen des minimierten Ausdrucks
% no       :  Anzahl der ODER-Verknpfungen des minimierten Ausdrucks

% Autor:  Klaus von der Heide, e-mail:  v.d.heide@on-line.de

ft   = binexp2bintab(binexp,variable);
fok  = all((ft==ftab)|(ftab<0));
nar  = sum(binexp=='&');
nor  = sum(binexp=='|');

s = [];

if fok & nar<=na & nor<=no  
   s = sprintf('korrekte Lsung!');
   if nar<na | nor<no  s = [s  sprintf('\ndie Lsung ist besser als die Vergleichslsung!')]; end
   Resultat = s;
   result2fig(Resultat)
   return
end

if ~fok
   if isdisjunctive(binexp)
      s = sprintf('Wenigstens ein Einspunkt liegt ausserhalb \noder ein Nullpunkt innerhalb der Schleifen!');
   elseif isconjunctive(binexp)
      s = sprintf('Wenigstens ein Nullpunkt liegt ausserhalb \noder ein Einspunkt innerhalb der Schleifen!');
   end
end

if nar>na | nor>no
   s = [s  sprintf('\nDer Ausdruck kann weiter minimiert werden!')];
end

Resultat = s;
result2fig(Resultat)



