function [ftabmin,rm,sing,t0] = minrm(ftab)
% [ftabmin,rm,sing,t0] = minrm(ftab)
% Minimierung des Realisierungsaufwandes fr eine Reed-Muller-Form.
% ftab    : Vektor der Funktionstabelle mit 0 und 1 sowie -1 fr Do'nt Care.
% ftabmin : realisierte Funktionstabelle
% rm      : Reed-Muller-Koeffizienten
% sing    : wenn 1 dann ist das Resultat unbrauchbar
% t0      : Indexvektor der zu 0 gemachten Terme
%
% !!! Zur Auflsung von Widersprchen und linearen Abhngigkeiten wird 
% !!! ein suboptimales Verfahren benutzt.
% !!! Dies ist eine eigentlich bessere Version von minrm, die aber noch
% !!! nich fehlerfrei ist.

% Autor:  Klaus von der Heide,  v.d.heide@on-line.de

% Default
% =======
if nargin<1  ftab =[1 -1  1  0  0  1  1 -1 -1  1 -1  0  0  1  0 -1]; end

% Konstanten
% ==========
nn = length(ftab);                 % Lnge der Funktionstabelle
n  = round(log2(nn));              % Anzahl der Variablen
ix = find(ftab<0);                 % Indices der Don't- Cares in ftab
m  = length(ix);                   % Anzahl der Don't-Cares

% Transformationsmatrix aufstellen
% ================================
k = nn;
A = 1;
Z = 0;
while k>1
   A = [A Z; A A];
   Z = [Z Z; Z Z];
   k = k/2;
end

% Transformation
% ==============
r  = rem(A*(ftab'>0),2)';          % Reed-Muller-Form mit Do'ntCares = 0
for k=1:length(ix)
   B(:,k) = A(:,ix(k));            % individuelle Transformation der Do'ntCares
end

% Aufwand der Terme
% =================
ii = find(any(B'));                % Indices der relevanten Terme
a  = sum(num2binmat((ii-1)',n)');  % Aufwand aller relevanten Terme
[dummy,jj] = sort(a);
iy = reverse(ii(jj));              % Termnummern sortiert nach sinkendem Aufwand
ra = reverse(a);                   % Aufwand der Terme

% Minimierung
% ===========
liy = length(iy);                  % Lnge der Liste der sortierten Terme
iz  = 1:m;
ftabmin = [];
rm   = [];
k    = 1;
sing = 1;
while sing
   t0 = iy(iz);                    % Indexvektor der Null-gesetzen Terme
   C  = B(t0,:);                   % Matrix des Gleichungssystems
   d  = r(t0)';                    % rechte Seite des Gleichungssystems
   try
      x  = bingausselim(C,d);      % Bestimmung der optimalen Do'ntCare-Werte
      ftabmin = ftab;              % feste Werte bernehmen
      ftabmin(ix) = x;             % berechnete Werte anstelle Do'ntCare einsetzen
      rm = rem(A*(ftabmin'>0),2)'; % Reed-Muller-Form ohne Do'ntCares
      sing = 0;
   catch
      for k=1:length(ix)
         ftb = ftab;
         ftb(ix(k)) = 0;           % Singularitt beseitigen durch Setzen einer Variablen
         [ftabmin0,rm0,sing,t0] = minrm3(ftb);
         ftb(ix(k)) = 1;           % 0 und 1 probieren
         [ftabmin1,rm1,sing,t0] = minrm3(ftb);
         if ~sing  
            if sum(rm0)<sum(rm1)   % prfen ob 0 besser als 1
               ftabmin = ftabmin0;
               rm = rm0;
            else
               ftabmin = ftabmin1;
               rm = rm1;
            end
            break;                 % beenden wenn erstmalig nicht singulr
         end
      end
   end
end
